package com.lejos.programmingbook.pongsimple;

import lejos.hardware.Brick;
import lejos.hardware.BrickFinder;
import lejos.hardware.Button;
import lejos.hardware.Sound;
import lejos.hardware.lcd.GraphicsLCD;
import lejos.hardware.lcd.LCD;
import lejos.hardware.motor.EV3LargeRegulatedMotor;
import lejos.hardware.port.MotorPort;
import lejos.utility.Delay;

/**
 * Implementation of Pong for EV3<P>
 * 
 * This implementation uses a simple programming style without introducing
 * separate classes for the different functionalities.
 * 
 * @author Alexander Schulze
 */
public class PongGameSimple {

	// define all required variables
	private int scorePlayer1 = 0;
	private int scorePlayer2 = 0;
	private int posY_Player1 = 0;
	private int posY_Player2 = 0;
	// ball
	private int posY = 0;
	private int posX = 0;
	private int vx = 0;
	private int vy = 0;

	private EV3LargeRegulatedMotor motorB = null;
	private EV3LargeRegulatedMotor motorC = null;

	private Brick ev3 = null;

	public PongGameSimple() {
		ev3 = BrickFinder.getDefault();
		motorB = new EV3LargeRegulatedMotor(MotorPort.B);
		motorC = new EV3LargeRegulatedMotor(MotorPort.C);
		motorB.resetTachoCount();
		motorC.resetTachoCount();
	}

	/**
	 * Initializes the ball position and velocity
	 */
	private void initBall() {
		posX = 89;
		posY = 64;
		vx = (int) (Math.random() * 7 - 4);
		if (vx == 0) {
			vx = 2;
		}
		vy = (int) (Math.random() * 5 - 3);
	}

	/**
	 * Paints the game screen containing score, ball, rackets
	 */
	private void paintGame() {
		LCD.clear();
		LCD.drawString(String.valueOf(scorePlayer1), 3, 0);
		LCD.drawString(String.valueOf(scorePlayer2), 12, 0);
		GraphicsLCD g = ev3.getGraphicsLCD();
		g.drawArc(posX, posY, 3, 3, 0, 360);
		g.drawLine(20, posY_Player1, 20, posY_Player1 + 20);
		g.drawLine(158, posY_Player2, 158, posY_Player2 + 20);
	}

	/**
	 * Checks for the value to be in a range of 0 to 108
	 * 
	 * @param value the value to check
	 * @return the value within the range or 0 or 108 if outside
	 */
	private int checkRange(int value) {
		if (value < 0) {
			return 0;
		} else if (value > 108) {
			return 108;
		}
		return value;
	}

	/**
	 * Calculates the position of the rackets based on the controller intput
	 */
	private void checkController() {
		posY_Player1 = checkRange(motorB.getTachoCount());
		posY_Player2 = checkRange(motorC.getTachoCount());
	}

	/**
	 * Tests if the ball is in contact with any of the two rackets and adjust the
	 * horizontal velocity accordingly or adds a point if the ball scored
	 * 
	 * @return <code>false</code> if a goal has been scored and the current game should end
	 */
	private boolean testRacket() {
		if (posX < 22) {
			if ((posY - posY_Player1 <= 20) && (posY - posY_Player1 >= 0)) {
				// ball contact to racket
				vx = vx * (-1);
			} else {
				scorePlayer2++;
				return false;
			}
		} else if (posX > 156) {
			if ((posY - posY_Player2 <= 20) && (posY - posY_Player2 >= 0)) {
				// ball contact to racket
				vx = vx * (-1);
			} else {
				scorePlayer1++;
				return false;
			}
		}
		return true;
	}

	/** 
	 * Moves the ball
	 */
	private void moveBall() {
		// ball movement
		posX = posX + vx;
		posY += vy;
		// test for ball contact at the top and the bottom
		if ((posY < 10) || (posY > 120)) {
			vy = vy * (-1);
		}
	}

	/**
	 * Main loop to play the game
	 */
	public void playGame() {
		boolean runGame = true;
		while (true) {
			runGame = true;
			initBall();
			paintGame();
			Button.waitForAnyPress();
			while (runGame) {
				moveBall();
				checkController();
				runGame = testRacket();
				paintGame();
				Delay.msDelay(100);
			}
			Sound.playTone(783, 1000);
		}

	}

	public static void main(String[] args) {

		PongGameSimple pong = new PongGameSimple();
		pong.playGame();

	}

}
