package de.lejos.fwk.motor;

import java.io.Closeable;

import lejos.hardware.motor.EV3LargeRegulatedMotor;
import lejos.hardware.port.MotorPort;
import lejos.hardware.port.Port;

/**
 * A differential wheeled robot is a mobile robot whose movement is based on two
 * separately driven wheels placed on either side of the robot body. It can thus
 * change its direction by varying the relative rate of rotation of its wheels
 * and hence does not require an additional steering motion (see Wikipedia).
 * 
 * This class represents a differential drive logic class providing convenient
 * methods to access the different motors.
 * 
 * @author Alexander Schulze
 *
 */
public class DifferentialDrive implements Closeable {

	private EV3LargeRegulatedMotor leftMotor;
	private EV3LargeRegulatedMotor rightMotor;

	private final static int DEFAULT_SPEED = 200;

	public DifferentialDrive(Port left_port, Port right_port, int speed) {
		leftMotor = new EV3LargeRegulatedMotor(left_port);
		rightMotor = new EV3LargeRegulatedMotor(right_port);

		leftMotor.setSpeed(speed);
		rightMotor.setSpeed(speed);
	}

	public DifferentialDrive(int speed) {
		this(MotorPort.B, MotorPort.C, speed);
	}

	public DifferentialDrive() {
		this(DEFAULT_SPEED);
	}

	public void backward() {
		leftMotor.backward();
		rightMotor.backward();
	}
	
	public void rotate(int rightAngle, int leftAngle) {
		leftMotor.rotate(leftAngle, true);
		rightMotor.rotate(rightAngle);
	}
	
	public void forward() {
		leftMotor.forward();
		rightMotor.forward();
	}

	public void stop() {
		leftMotor.stop(true);
		rightMotor.stop();
	}

	public void rotateClockwise() {
		leftMotor.forward();
		rightMotor.backward();
	}

	public void rotateCounterClockwise() {
		leftMotor.backward();
		rightMotor.forward();
	}

	public void setSpeed(int speed) {
		leftMotor.setSpeed(speed);
		rightMotor.setSpeed(speed);
	}

	public void left() {
		leftMotor.stop(true);
		rightMotor.forward();
	}

	public void right() {
		rightMotor.stop(true);
		leftMotor.forward();
	}

	public boolean isStopped() {
		return !rightMotor.isMoving() && !leftMotor.isMoving();
	}

	@Override
	public void close() {
		rightMotor.close();
		leftMotor.close();
	}
	
	public int getRightMotorTachoCount() {
		return rightMotor.getTachoCount();
	}

	public int getLeftMotorTachoCount() {
		return leftMotor.getTachoCount();
	}
	
}
